@@ -18,7 +18,7 @@
 #include "lldpd.h"
 #include "trace.h"
 
-static int
+static ssize_t
 client_handle_none(struct lldpd *cfg, enum hmsg_type *type,
     void *input, int input_len, void **output, int *subscribed)
 {
@@ -28,7 +28,7 @@ client_handle_none(struct lldpd *cfg, enum hmsg_type *type,
 }
 
 /* Return the global configuration */
-static int
+static ssize_t
 client_handle_get_configuration(struct lldpd *cfg, enum hmsg_type *type,
     void *input, int input_len, void **output, int *subscribed)
 {
@@ -42,8 +42,15 @@ client_handle_get_configuration(struct lldpd *cfg, enum hmsg_type *type,
 	return output_len;
 }
 
+static char*
+xstrdup(const char *str)
+{
+	if (!str) return NULL;
+	return strdup(str);
+}
+
 /* Change the global configuration */
-static int
+static ssize_t
 client_handle_set_configuration(struct lldpd *cfg, enum hmsg_type *type,
     void *input, int input_len, void **output, int *subscribed)
 {
@@ -55,35 +62,41 @@ client_handle_set_configuration(struct lldpd *cfg, enum hmsg_type *type,
 		*type = NONE;
 		return 0;
 	}
-	
+
+#define CHANGED(w) (config->w != cfg->g_config.w)
+#define CHANGED_STR(w) (!(config->w == cfg->g_config.w ||	\
+		(config->w && cfg->g_config.w && !strcmp(config->w, cfg->g_config.w))))
+
 	/* What needs to be done? Transmit delay? */
-	if (config->c_tx_interval > 0) {
-		log_debug("rpc", "client change transmit interval to %d",
-			config->c_tx_interval);
-		cfg->g_config.c_tx_interval = config->c_tx_interval;
-		LOCAL_CHASSIS(cfg)->c_ttl = cfg->g_config.c_tx_interval *
-			cfg->g_config.c_tx_hold;
+	if (CHANGED(c_tx_interval) && config->c_tx_interval != 0) {
+		if (config->c_tx_interval < 0) {
+			log_debug("rpc", "client asked for immediate retransmission");
+			levent_send_now(cfg);
+		} else {
+			log_debug("rpc", "client change transmit interval to %d",
+			    config->c_tx_interval);
+			cfg->g_config.c_tx_interval = config->c_tx_interval;
+			LOCAL_CHASSIS(cfg)->c_ttl = cfg->g_config.c_tx_interval *
+			    cfg->g_config.c_tx_hold;
+		}
 	}
-	if (config->c_tx_hold > 0) {
+	if (CHANGED(c_tx_hold) && config->c_tx_hold > 0) {
 		log_debug("rpc", "client change transmit hold to %d",
-			config->c_tx_hold);
+		    config->c_tx_hold);
 		cfg->g_config.c_tx_hold = config->c_tx_hold;
 		LOCAL_CHASSIS(cfg)->c_ttl = cfg->g_config.c_tx_interval *
-			cfg->g_config.c_tx_hold;
+		    cfg->g_config.c_tx_hold;
 	}
-	if (config->c_tx_interval < 0) {
-		log_debug("rpc", "client asked for immediate retransmission");
-		levent_send_now(cfg);
-	}
-	if (config->c_lldp_portid_type > LLDP_PORTID_SUBTYPE_UNKNOWN &&
-            config->c_lldp_portid_type <= LLDP_PORTID_SUBTYPE_MAX) {
-            log_debug("rpc", "change lldp portid tlv subtype to %d",
-                      config->c_lldp_portid_type);
-            cfg->g_config.c_lldp_portid_type = config->c_lldp_portid_type;
-	    levent_update_now(cfg);
+	if (CHANGED(c_lldp_portid_type) &&
+	    config->c_lldp_portid_type > LLDP_PORTID_SUBTYPE_UNKNOWN &&
+	    config->c_lldp_portid_type <= LLDP_PORTID_SUBTYPE_MAX) {
+		log_debug("rpc", "change lldp portid tlv subtype to %d",
+		    config->c_lldp_portid_type);
+		cfg->g_config.c_lldp_portid_type = config->c_lldp_portid_type;
+		levent_update_now(cfg);
 	}
 	/* Pause/resume */
-	if (config->c_paused != cfg->g_config.c_paused) {
+	if (CHANGED(c_paused)) {
 		log_debug("rpc", "client asked to %s lldpd",
 		    config->c_paused?"pause":"resume");
 		cfg->g_config.c_paused = config->c_paused;
@@ -91,59 +104,65 @@ client_handle_set_configuration(struct lldpd *cfg, enum hmsg_type *type,
 	}
 
 #ifdef ENABLE_LLDPMED
-	if (config->c_enable_fast_start) {
-		cfg->g_config.c_enable_fast_start = (config->c_enable_fast_start == 1);
+	if (CHANGED(c_enable_fast_start)) {
+		cfg->g_config.c_enable_fast_start = config->c_enable_fast_start;
 		log_debug("rpc", "client asked to %s fast start",
 		    cfg->g_config.c_enable_fast_start?"enable":"disable");
 	}
-	if (config->c_tx_fast_interval) {
+	if (CHANGED(c_tx_fast_interval) &&
+	    config->c_tx_fast_interval > 0) {
 		log_debug("rpc", "change fast interval to %d", config->c_tx_fast_interval);
 		cfg->g_config.c_tx_fast_interval = config->c_tx_fast_interval;
 	}
 #endif
-	if (config->c_iface_pattern) {
-		log_debug("rpc", "change interface pattern to %s", config->c_iface_pattern);
+	if (CHANGED_STR(c_iface_pattern)) {
+		log_debug("rpc", "change interface pattern to %s",
+		    config->c_iface_pattern?config->c_iface_pattern:"(NULL)");
 		free(cfg->g_config.c_iface_pattern);
-		cfg->g_config.c_iface_pattern = strdup(config->c_iface_pattern);
+		cfg->g_config.c_iface_pattern = xstrdup(config->c_iface_pattern);
 		levent_update_now(cfg);
 	}
-	if (config->c_mgmt_pattern) {
-		log_debug("rpc", "change management pattern to %s", config->c_mgmt_pattern);
+	if (CHANGED_STR(c_mgmt_pattern)) {
+		log_debug("rpc", "change management pattern to %s",
+		    config->c_mgmt_pattern?config->c_mgmt_pattern:"(NULL)");
 		free(cfg->g_config.c_mgmt_pattern);
-		cfg->g_config.c_mgmt_pattern = strdup(config->c_mgmt_pattern);
+		cfg->g_config.c_mgmt_pattern = xstrdup(config->c_mgmt_pattern);
 		levent_update_now(cfg);
 	}
-	if (config->c_description) {
-		log_debug("rpc", "change chassis description to %s", config->c_description);
+	if (CHANGED_STR(c_description)) {
+		log_debug("rpc", "change chassis description to %s",
+		    config->c_description?config->c_description:"(NULL)");
 		free(cfg->g_config.c_description);
-		cfg->g_config.c_description = strdup(config->c_description);
+		cfg->g_config.c_description = xstrdup(config->c_description);
 		levent_update_now(cfg);
 	}
-	if (config->c_platform) {
-		log_debug("rpc", "change platform description to %s", config->c_platform);
+	if (CHANGED_STR(c_platform)) {
+		log_debug("rpc", "change platform description to %s",
+		    config->c_platform?config->c_platform:"(NULL)");
 		free(cfg->g_config.c_platform);
-		cfg->g_config.c_platform = strdup(config->c_platform);
+		cfg->g_config.c_platform = xstrdup(config->c_platform);
 		levent_update_now(cfg);
 	}
-	if (config->c_hostname) {
-		log_debug("rpc", "change system name to %s", config->c_hostname);
+	if (CHANGED_STR(c_hostname)) {
+		log_debug("rpc", "change system name to %s",
+		    config->c_hostname?config->c_hostname:"(NULL");
 		free(cfg->g_config.c_hostname);
-		cfg->g_config.c_hostname = strdup(config->c_hostname);
+		cfg->g_config.c_hostname = xstrdup(config->c_hostname);
 		levent_update_now(cfg);
 	}
-	if (config->c_set_ifdescr != cfg->g_config.c_set_ifdescr) {
+	if (CHANGED(c_set_ifdescr)) {
 		log_debug("rpc", "%s setting of interface description based on discovered neighbors",
 		    config->c_set_ifdescr?"enable":"disable");
 		cfg->g_config.c_set_ifdescr = config->c_set_ifdescr;
 		levent_update_now(cfg);
 	}
-	if (config->c_promisc != cfg->g_config.c_promisc) {
+	if (CHANGED(c_promisc)) {
 		log_debug("rpc", "%s promiscuous mode on managed interfaces",
 		    config->c_promisc?"enable":"disable");
 		cfg->g_config.c_promisc = config->c_promisc;
 		levent_update_now(cfg);
 	}
-	if (config->c_bond_slave_src_mac_type != 0) {
+	if (CHANGED(c_bond_slave_src_mac_type)) {
 		if (config->c_bond_slave_src_mac_type >
 		    LLDP_BOND_SLAVE_SRC_MAC_TYPE_UNKNOWN &&
 		    config->c_bond_slave_src_mac_type <=
@@ -168,13 +187,13 @@ client_handle_set_configuration(struct lldpd *cfg, enum hmsg_type *type,
    Input:  nothing.
    Output: list of interface names (lldpd_interface_list)
 */
-static int
+static ssize_t
 client_handle_get_interfaces(struct lldpd *cfg, enum hmsg_type *type,
     void *input, int input_len, void **output, int *subscribed)
 {
 	struct lldpd_interface *iff, *iff_next;
 	struct lldpd_hardware *hardware;
-	int output_len;
+	ssize_t output_len;
 
 	/* Build the list of interfaces */
 	struct lldpd_interface_list ifs;
@@ -211,7 +230,7 @@ client_handle_get_interfaces(struct lldpd *cfg, enum hmsg_type *type,
    Input:  name of the interface (serialized)
    Output: Information about the interface (lldpd_hardware)
 */
-static int
+static ssize_t
 client_handle_get_interface(struct lldpd *cfg, enum hmsg_type *type,
     void *input, int input_len, void **output, int *subscribed)
 {
@@ -230,7 +249,7 @@ client_handle_get_interface(struct lldpd *cfg, enum hmsg_type *type,
 	log_debug("rpc", "client request interface %s", name);
 	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries)
 		if (!strcmp(hardware->h_ifname, name)) {
-			int output_len = lldpd_hardware_serialize(hardware, output);
+			ssize_t output_len = lldpd_hardware_serialize(hardware, output);
 			free(name);
 			if (output_len <= 0) {
 				*type = NONE;
@@ -249,7 +268,7 @@ client_handle_get_interface(struct lldpd *cfg, enum hmsg_type *type,
    Input: name of the interface, policy/location/power setting to be modified
    Output: nothing
 */
-static int
+static ssize_t
 client_handle_set_port(struct lldpd *cfg, enum hmsg_type *type,
     void *input, int input_len, void **output, int *subscribed)
 {
@@ -350,7 +369,7 @@ set_port_finished:
 }
 
 /* Register subscribtion to neighbor changes */
-static int
+static ssize_t
 client_handle_subscribe(struct lldpd *cfg, enum hmsg_type *type,
     void *input, int input_len, void **output, int *subscribed)
 {
@@ -362,7 +381,7 @@ client_handle_subscribe(struct lldpd *cfg, enum hmsg_type *type,
 struct client_handle {
 	enum hmsg_type type;
 	const char *name;
-	int (*handle)(struct lldpd*, enum hmsg_type *,
+	ssize_t (*handle)(struct lldpd*, enum hmsg_type *,
 	    void *, int, void **, int *);
 };
 
@@ -384,7 +403,7 @@ client_handle_client(struct lldpd *cfg,
     int *subscribed)
 {
 	struct client_handle *ch;
-	void *answer; size_t len, sent;
+	void *answer; ssize_t len, sent;
 
 	log_debug("rpc", "handle client request");
 	for (ch = client_handles; ch->handle != NULL; ch++) {
