@@ -207,6 +207,7 @@ lldpd_hardware_cleanup(struct lldpd *cfg, struct lldpd_hardware *hardware)
 {
 	log_debug("alloc", "cleanup hardware port %s", hardware->h_ifname);
 
+	free(hardware->h_lport_previous);
 	lldpd_port_cleanup(&hardware->h_lport, 1);
 	if (hardware->h_ops && hardware->h_ops->cleanup)
 		hardware->h_ops->cleanup(cfg, hardware);
@@ -292,14 +293,13 @@ lldpd_reset_timer(struct lldpd *cfg)
 	/* Reset timer for ports that have been changed. */
 	struct lldpd_hardware *hardware;
 	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries) {
-		/* We need to compute a checksum of the local port. To do this,
-		 * we zero out fields that are not significant, marshal the
-		 * port, compute the checksum, then restore. */
+		/* We keep a flat copy of the local port to see if there is any
+		 * change. To do this, we zero out fields that are not
+		 * significant, marshal the port, then restore. */
 		struct lldpd_port *port = &hardware->h_lport;
-		u_int16_t cksum;
 		u_int8_t *output = NULL;
-		size_t output_len;
-		char save[offsetof(struct lldpd_port, p_id_subtype)];
+		ssize_t output_len;
+		char save[LLDPD_PORT_START_MARKER];
 		memcpy(save, port, sizeof(save));
 		/* coverity[suspicious_sizeof]
 		   We intentionally partially memset port */
@@ -312,19 +312,25 @@ lldpd_reset_timer(struct lldpd *cfg)
 			    hardware->h_ifname);
 			continue;
 		}
-		cksum = frame_checksum(output, output_len, 0);
-		free(output);
-		if (cksum != hardware->h_lport_cksum) {
-			log_debug("localchassis",
-			    "change detected for port %s, resetting its timer",
-			    hardware->h_ifname);
-			hardware->h_lport_cksum = cksum;
-			levent_schedule_pdu(hardware);
-		} else {
+
+		/* Compare with the previous value */
+		if (hardware->h_lport_previous &&
+		    output_len == hardware->h_lport_previous_len &&
+		    !memcmp(output, hardware->h_lport_previous, output_len)) {
 			log_debug("localchassis",
 			    "no change detected for port %s",
 			    hardware->h_ifname);
+		} else {
+			log_debug("localchassis",
+			    "change detected for port %s, resetting its timer",
+			    hardware->h_ifname);
+			levent_schedule_pdu(hardware);
 		}
+
+		/* Update the value */
+		free(hardware->h_lport_previous);
+		hardware->h_lport_previous = output;
+		hardware->h_lport_previous_len = output_len;
 	}
 }
 
@@ -1284,9 +1290,9 @@ lldpd_main(int argc, char *argv[], char *envp[])
 	int ch, debug = 0;
 #ifdef USE_SNMP
 	int snmp = 0;
-	char *agentx = NULL;	/* AgentX socket */
+	const char *agentx = NULL;	/* AgentX socket */
 #endif
-	char *ctlname = LLDPD_CTL_SOCKET;
+	const char *ctlname = NULL;
 	char *mgmtp = NULL;
 	char *cidp = NULL;
 	char *interfaces = NULL;
@@ -1353,7 +1359,7 @@ lldpd_main(int argc, char *argv[], char *envp[])
 				fprintf(stderr, "-m can only be used once\n");
 				usage();
 			}
-			mgmtp = optarg;
+			mgmtp = strdup(optarg);
 			break;
 		case 'u':
 			if (ctlname) {
@@ -1367,14 +1373,14 @@ lldpd_main(int argc, char *argv[], char *envp[])
 				fprintf(stderr, "-I can only be used once\n");
 				usage();
 			}
-			interfaces = optarg;
+			interfaces = strdup(optarg);
 			break;
 		case 'C':
 			if (cidp) {
 				fprintf(stderr, "-C can only be used once\n");
 				usage();
 			}
-			cidp = optarg;
+			cidp = strdup(optarg);
 			break;
 		case 'L':
 			if (strlen(optarg)) lldpcli = optarg;
@@ -1450,6 +1456,8 @@ lldpd_main(int argc, char *argv[], char *envp[])
 		}
 	}
 
+	if (ctlname == NULL) ctlname = LLDPD_CTL_SOCKET;
+
 	/* Set correct smart mode */
 	for (i=0; (filters[i].a != -1) && (filters[i].a != smart); i++);
 	if (filters[i].a == -1) {
