@@ -225,8 +225,8 @@ struct lldpd_port {
 	u_int8_t		 p_hidden_in:1; /* Considered as hidden for reception */
 	u_int8_t		 p_hidden_out:2; /* Considered as hidden for emission */
 	/* Important: all fields that should be ignored to check if a port has
-	 * been changed should be before p_id_subtype. Check
-	 * `lldpd_reset_timer()`. */
+	 * been changed should be before this mark. */
+#define LLDPD_PORT_START_MARKER (offsetof(struct lldpd_port, p_id_subtype))
 	u_int8_t		 p_id_subtype;
 	char			*p_id;
 	int			 p_id_len;
@@ -398,7 +398,8 @@ struct lldpd_hardware {
 	u_int64_t		 h_delete_cnt;
 	u_int64_t		 h_drop_cnt;
 
-	u_int16_t		 h_lport_cksum; /* Checksum on local port to see if there is a change */
+	void			*h_lport_previous; /* Backup of last value for localport */
+	ssize_t			 h_lport_previous_len;
 	struct lldpd_port	 h_lport;  /* Port attached to this hardware port */
 	TAILQ_HEAD(, lldpd_port) h_rports; /* Remote ports */
 
